module RangeDict exposing (RangeDict, any, empty, insert, remove)

import Dict exposing (Dict)
import Elm.Syntax.Range exposing (Range)


type RangeDict value
    = RangeDict (Dict ( ( Int, Int ), ( Int, Int ) ) value)


empty : RangeDict v_
empty =
    RangeDict Dict.empty


rangeToTuple : Range -> ( ( Int, Int ), ( Int, Int ) )
rangeToTuple =
    \range ->
        ( ( range.start.row, range.start.column )
        , ( range.end.row, range.end.column )
        )


insert : ( Range, value ) -> (RangeDict value -> RangeDict value)
insert ( range, value ) =
    \(RangeDict rangeDict) ->
        RangeDict (Dict.insert (rangeToTuple range) value rangeDict)


remove : Range -> (RangeDict value -> RangeDict value)
remove range =
    \(RangeDict rangeDict) ->
        RangeDict (Dict.remove (rangeToTuple range) rangeDict)


any : (( Range, value ) -> Bool) -> RangeDict value -> Bool
any isFound =
    \rangeDict ->
        foldl (\( range, value ) soFar -> soFar || isFound ( range, value ))
            False
            rangeDict


foldl : (( Range, value ) -> folded -> folded) -> folded -> RangeDict value -> folded
foldl reduce initialFolded =
    \(RangeDict rangeDict) ->
        rangeDict
            |> Dict.foldl (\tuple value -> reduce ( tuple |> tupleToRange, value )) initialFolded


tupleToRange : ( ( Int, Int ), ( Int, Int ) ) -> Range
tupleToRange =
    \( ( startRow, startColumn ), ( endRow, endColumn ) ) ->
        { start = { row = startRow, column = startColumn }
        , end = { row = endRow, column = endColumn }
        }
